#!/bin/env python
import time
import parmepy as pp
import math
import sys


def run(nb=257):
    dim = 3
    boxLength = [1., 1., 1.]
    boxMin = [0., 0., 0.]
    nbElem = [nb, nb, nb]

    timeStep = 0.05
    #period = 10.
    finalTime = 40*timeStep
    outputFilePrefix = './res_3D/levelSet_3D_'
    outputModulo = 0

    t0 = time.time()

    ## Domain
    box = pp.Box(dim, length=boxLength, origin=boxMin)

    ## Fields
    scal = pp.ContinuousField(domain=box, name='Scalar')
    velo = pp.ContinuousField(domain=box, name='Velocity', vector=True)
    #scal = pp.AnalyticalField(domain=box, formula=scalaire, name='Scalar')
    #velo = pp.AnalyticalField(domain=box, formula=vitesse, name='Velocity', vector=True)

    ## Operators
    advec = pp.Transport(velo, scal)
    #velocity = pp.Velocity(velo)

    ## Solver creation (discretisation of objects is done in solver initialisation)
    topo3D = pp.CartesianTopology(domain=box, resolution=nbElem, dim=dim)

    ##Problem
    #pb = pp.Problem(topo3D, [velocity, advec])
    pb = pp.Problem(topo3D, [advec])

    ## Setting solver to Problem
    pb.setSolver(finalTime, timeStep, 'gpu',
                 src=['./levelSet3D.cl'],
                 io=pp.Printer(fields=[scal], frequency=outputModulo, outputPrefix=outputFilePrefix),
                 precision=pp.constants.PARMES_REAL_GPU
                 #precision=pp.constants.PARMES_DOUBLE_GPU
                 )
    pb.initSolver()

    t1 = time.time()

    ## Solve problem
    timings = pb.solve()

    tf = time.time()

    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"
    print ""
    return pb.timings_info[0] + "\"Solving time\" \"Init time\" \"Total time\"", pb.timings_info[1] + str(tf - t1) + " " + str(t1 - t0) + " " + str(tf - t0)

if __name__ == "__main__":
    run(257)
    # timings = {}
    # f = open('bench_levelSet3D.dat', 'w')
    # header = "#size dim nPart "
    # sizes = xrange(32, 256+32*3+1, 32)
    # #sizes = [256+32+32+32]
    # for s in sizes:
    #     h, t = run(s)
    #     timings[s] = t
    # header += h
    # f.write(header + "\n")
    # for s in sizes:
    #     f.write(str(s) + " " + str(3) + " " + str(s**3) + " " + str(timings[s]) + "\n")
    # f.close()
