# -*- coding: utf-8 -*-
import time
import parmepy as pp
from math import *


def vitesse(x, y, z):
    vx = 1. + x
    vy = - x * y
    vz = x * y * z + 10.
    return vx, vy, vz


def scalaire(x, y, z):
    if x < 0.5 and y < 0.5 and z < 0.5:
        return 1.
    else:
        return 0.


def run():
    # Parameters
    nb = 256
    timeStep = 0.02
    finalTime = 1.
    outputFilePrefix = './res/RK2_'
    outputModulo = 0

    t0 = time.time()

    ## Domain
    box = pp.Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])

    ## Fields
    scal = pp.ContinuousField(domain=box, name='Scalar')
    velo = pp.ContinuousField(domain=box, name='Velocity', vector=True)
    #scal = pp.AnalyticalField(domain=box, name='Scalar')
    #velo = pp.AnalyticalField(domain=box, formula=vitesse, name='Velocity', vector=True)

    ## Operators
    advec = pp.Transport(velo, scal)

    ## Solver creation (discretisation of objects is done in solver initialisation)
    topo3D = pp.CartesianTopology(domain=box, resolution=[nb, nb, nb], dim=3)

    ##Problem
    pb = pp.Problem(topo3D, [advec])

    ## Setting solver to Problem
    #pb.setSolver(finalTime, timeStep, 'gpu',
    pb.setSolver(finalTime, timeStep, 'gpu',src='./mainJM_kernels.cl',
                 io=pp.Printer(fields=[scal, velo], frequency=outputModulo, outputPrefix=outputFilePrefix))
    pb.initSolver()

    t1 = time.time()

    ## Solve problem
    pb.solve()

    tf = time.time()

    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    run()
