"""
@package parmepy

Python package dedicated to flow simulation using particular methods on hybrid architectures (MPI-GPU)

"""
__version__ = 1.00
__all__ = ['Box', 'CartesianTopology', 'ScalarField']

import os
import site
import mpi4py.MPI as MPI
import pyopencl as cl

rank_world = MPI.COMM_WORLD.Get_rank()
if(rank_world == 0):
    print "Starting Parmes version " + str(__version__) + ".\n"

version = "1.0.0"

import constants

import domain.box
## Box-type physical domain
Box = domain.box.Box

## Cartesian grid

## MPI topologies and associated meshes
import domain.topology
CartesianTopology = domain.topology.CartesianTopology

## Fields
import fields.discrete
import fields.continuous
import fields.analytical
ScalarField = fields.discrete.ScalarField
VectorField = fields.discrete.VectorField
ContinuousField = fields.continuous.ContinuousField
AnalyticalField = fields.analytical.AnalyticalField

## Obstacle
import obstacle.obstacle
import obstacle.obstacle_d
Obstacle = obstacle.obstacle.Obstacle
Obstacle_d = obstacle.obstacle_d.Obstacle_d

## Operators
import operator.advec_scales
import operator.diffusion
import operator.penalization
import operator.poisson
import operator.stretching
import operator.transport
import operator.velocity
Advec_scales = operator.advec_scales.Advec_scales
Diffusion = operator.diffusion.Diffusion
Penalization = operator.penalization.Penalization
Poisson = operator.poisson.Poisson
Stretching = operator.stretching.Stretching
Transport = operator.transport.Transport
Velocity = operator.velocity.Velocity


## Problem
import problem.problem
Problem = problem.problem.Problem


## Forces
import physics.compute_forces
Compute_forces = physics.compute_forces.Compute_forces

## Solver
import particular_solvers.basic
import particular_solvers.gpu
ParticleSolver = particular_solvers.basic.ParticleSolver
GPUParticleSolver = particular_solvers.gpu.GPUParticleSolver

## Tools
import tools.printer
Printer = tools.printer.Printer

from tools.explore_hardware import explore
