"""@package parmepy.domain.grid

Cartesian grid definition.
"""
from ..constants import *
from .discrete import DiscreteDomain


class CartesianGrid(DiscreteDomain):
    """ Discrete box representation as a cartesian grid."""

    def __init__(self, resolution, domain):
        """
        Creates a CartesianGrid.

        @param resolution : resolution used
        @param domain : Domain to discretize
        """
        DiscreteDomain.__init__(self, domain)
        assert(self.dimension == len(resolution))
        ## lowest point of the grid
        self.origin = domain.origin
        ## size of the grid in each direction
        self.length = domain.length
        ## number of points in each direction
        self.resolution = resolution
        ## space step size
        self.step = self.length / (self.resolution)

    def update(start):
        self.start = start

    def __getitem__(self, index):
        """
        Get iter overriding

        @param index list : index to get
        @return point with given index
        """
        return self.__coords.__getitem__(index)

    def __str__(self):
        """
        ToString method.
        """
        s = str(self.dimension) + "D cartesian grid with resolution equal to : " + str(self.resolution) + ".\n"
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : CartesianGrid"
    print CartesianGrid.__doc__
