"""
@package parmepy.fields.continuous

Continuous variable description
"""
from .discrete import ScalarField
from .discrete import VectorField


class ContinuousField(object):
    """ A variable defined on a physical domain """

    def __init__(self, domain, name="?", vector=False):
        """
        Create a ContinuousField.

        @param domain : variable application domain.
        @param name : name of the variable (used for vtk output).
        @param vector : is variable is a vector.
        """
        ## Application domain of the variable.
        self.domain = domain
        ## Field dimension.
        self.dimension = self.domain.dimension
        ## Discretisation of this field
        self.discreteField = None
        ## Name of this field
        self.name = name
        # Number of different discretizations
        self._fieldId = -1
        ## List of the various discretizations of this field
        self.discreteField = []
        ## Is field is a vector field
        self.vector = vector

    def discretize(self, topology=None):
        """
        Discretization of the field on a topology
        @param topology : Topology definition to use
        @return discrete field and its index
        """
        self._fieldId += 1
        if self.vector:
            self.discreteField.append(VectorField(self, topology,
                                                  name=self.name + "_D_" + str(self._fieldId),
                                                  idFromParent=self._fieldId))
        else:
            self.discreteField.append(ScalarField(self, topology,
                                                  name=self.name + "_D_" + str(self._fieldId),
                                                  idFromParent=self._fieldId))
        return self.discreteField[self._fieldId], self._fieldId

    def initialize(self):
        """
        Initialize a field.
        @see VectorField.initialize
        @see ScalarField.initialize
        """
        if self._fieldId == -1:
            raise ValueError("Cannot initialise analytical field non discretized.")
        print self.name,
        for dF in self.discreteField:
            dF.initialize()
        print " .Done"

    def __str__(self):
        """ Field info display """
        s = self.name + ' , ' + str(self.dimension) + 'D field '
        if len(self.discreteField) != 0:
            s += "with the following discretizations:\n"
            for i in range(len(self.discreteField)):
                s += self.discreteField[i].__str__()
        else:
            s += ". Not discretised\n"
        return s


if __name__ == "__main__":
    print __doc__
    print "- Provided class : ContinuousField"
    print ContinuousField.__doc__
