# -*- coding: utf-8 -*-
"""
@package parmepy.operator Advec_scales
Continous Advection operator using Scales code (Jean-Baptiste)
"""
from .continuous import ContinuousOperator
from .advec_scales_d import Advec_scales_d


class Advec_scales(ContinuousOperator):
    """
    Advection scales operator representation
    """

    def __init__(self, stab_coeff=None, velocity=None, advectedField=None):
        """
        Constructor.
        Create an Advection operator using scales code (JB) from given velocity variables.

        @param velocity ContinuousVectorField : velocity variable.
        @param advectedField ContinuousVector/ScalarField : variable to advect.
        """
        ContinuousOperator.__init__(self)
        self.stab_coeff = stab_coeff
        self.velocity = velocity
        self.advectedField = advectedField
        self.addVariable([velocity, advectedField])

    def discretize(self, idVelocityD=0, idAdvectedFieldD=0, topology=None):
        """
        Advection operator discretization method.
        Create a discrete advection operator from given specifications.

        @param idVelocityD : Index of velocity discretisation to use for advection.
        @param idAdvectedFieldD : Index of field discretisation to advect.
        @param topology : topology of the input fields
        """
        if self.discreteOperator is None:
            self.discreteOperator = Advec_scales_d(self, idVelocityD, idAdvectedFieldD, self.stab_coeff, topology)

    def __str__(self):
        """ToString method"""
        s = " Advection scales operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Advec_scales"
    print Advec_scales.__doc__
