# -*- coding: utf-8 -*-
"""
@package operator
Discrete Advection operator using scales code (Jean-Baptiste)
"""
from ..f2py import scales2py
import mpi4py.MPI as MPI
from .discrete import DiscreteOperator
from ..constants import *
import numpy as np
import time
import sys

class Advec_scales_d(DiscreteOperator):
    """
    Operator representation.
    """

    def __init__(self, advec, idVelocityD=0, idAdvectedFieldD=0, stab_coeff=None, topology=None):
        """
        Constructor.
        @param stab_coeff : stability coefficient
        @param velocity : descretized velocity to use.
        @param advectedField : discretized field to advect.
        """
        DiscreteOperator.__init__(self)
        self.advec = advec
        self.velocity = advec.velocity.discreteField[idVelocityD]
        self.advectedField = advec.advectedField.discreteField[idAdvectedFieldD]
        self.stab_coeff = stab_coeff
        self.topology = topology
        self.ghosts = topology.ghosts
        self.resolution = topology.mesh.resolution
        self.dim = topology.dim
#        self.scales = parmepy.f2py.scales2py
        self.compute_time = 0.

    def apply(self, t, dt):
        """
        Apply operator.
        """
        self.compute_time = time.time()

        ind0a = self.ghosts[0]
        ind0b=self.resolution[0]-self.ghosts[0]
        ind1a = self.ghosts[1]
        ind1b=self.resolution[1]-self.ghosts[1]
        ind2a = self.ghosts[2]
        ind2b=self.resolution[2]-self.ghosts[2]
        ind0=np.arange(ind0a,ind0b)
        ind1=np.arange(ind1a,ind1b)
        ind2=np.arange(ind2a,ind2b)

        ndt = 1
        if (dt % self.stab_coeff == 0):
            ndt = int(dt / self.stab_coeff)
        else :
            ndt = int(dt / self.stab_coeff) + 1

        dtadvec = dt/float(ndt)
        for i in range (ndt) :
            print 'advection cycle n°', i
            if self.advectedField.vector :

                advecFieldNoG = [np.zeros((self.resolution - 2 * self.ghosts), dtype=PARMES_REAL, order=ORDER) for d in xrange(self.dim)]
                velocityNoG = [np.zeros((self.resolution - 2 * self.ghosts), dtype=PARMES_REAL, order=ORDER) for d in xrange(self.dim)]
                for i in xrange (self.dim):
                    advecFieldNoG[i] = self.advectedField[i][ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]
                    velocityNoG[i] = self.velocity[i][ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]
                print 'shape vortiNoG, veloNoG', np.asarray(advecFieldNoG).shape, np.asarray(velocityNoG).shape

                # Vector field advection
                advecFieldNoG[0][...] = scales2py.solve_advection(dtadvec, velocityNoG[0][...], \
                                             velocityNoG[1][...], velocityNoG[2][...], advecFieldNoG[0][...])
                advecFieldNoG[1][...] = scales2py.solve_advection(dtadvec, velocityNoG[0][...], \
                                             velocityNoG[1][...], velocityNoG[2][...], advecFieldNoG[1][...])
                advecFieldNoG[2][...] = scales2py.solve_advection(dtadvec, velocityNoG[0][...], \
                                             velocityNoG[1][...], velocityNoG[2][...], advecFieldNoG[2][...])

                for i in xrange (self.dim):
                    self.advectedField[i][ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = advecFieldNoG[i][...]
                print 'shape vorti, velo', np.asarray(self.advectedField.data).shape, np.asarray(self.velocity.data).shape

            else :

                advecFieldNoG = self.advectedField[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]
                velocityNoG = self.velocity[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]
                # Scalar field advection
                advecFieldNoG[...] = scales2py.solve_advection(dtadvec, velocityNoG[0][...], \
                                          velocityNoG[1][...], velocityNoG[2][...], advecFieldNoG[...])

                self.advectedField[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = advecFieldNoG[...]

        self.compute_time = time.time() - self.compute_time
        self.total_time += self.compute_time
        return self.compute_time

    def printComputeTime(self):
        self.timings_info[0] = "\"Advection calculation total\""
        self.timings_info[1] = str(self.total_time)
#        print "Advection scales total time : ", self.total_time
        print "Time of the last advection calculation loop :", self.compute_time 

    def __str__(self):
        s = "Advec_scales_d (DiscreteOperator). " + DiscreteOperator.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Advec_scales_d"
    print Advec_scales_d.__doc__
