"""
@package parmepy.operator.continuous

Operator representation.
"""
from abc import ABCMeta, abstractmethod


class ContinuousOperator:
    """
    Continuous operator abstract description.
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self):
        """
        Create a ContinuousOperator.
        """
        ## Variables
        self.variables = []
        ## Operator discretization.
        self.discreteOperator = None
        ## Is need to split operator
        self.needSplitting = False
        ## Timings informations
        self.timings_info = ["", ""]

    def addVariable(self, cVariable):
        """
        Add an continuous variables to the operator.

        @param cVariable : list of variables to add.
        """
        for v in cVariable:
            if self.variables.count(v) == 0:
                self.variables.append(v)

    def apply(self, *args):
        """
        Apply operator.

        @param *args : Operator arguments.
        @return computing time.
        """
        return self.discreteOperator.apply(*args)

    def setMethod(self, method):
        """
        Sets method to use in apply method.
        @param method : the method to use.
        """
        if self.discreteOperator is not None:
            self.discreteOperator.setMethod(method)
        else:
            raise ValueError("Cannot set mumerical method to non discretized operator")

    def printComputeTime(self):
        """Displays compute time for operator."""
        if self.discreteOperator is not None:
            self.discreteOperator.printComputeTime()
            self.timings_info = self.discreteOperator.timings_info
        else:
            raise ValueError("Cannot print compute time of a non discretized operator")

    @abstractmethod
    def discretize(self, *spec):
        """
        Abstract method.
        Must be implemented by sub-class.

        @param *spec : discretization specifications.
        """
        raise NotImplementedError("Need to override method in a subclass of ContinuousOperator")

if __name__ == "__main__":
    print __doc__
    print "- Provided class : ContinuousOperator"
    print ContinuousOperator.__doc__
