# -*- coding: utf-8 -*-
"""
@package parmepy.operator.Stretching

Penalization operator representation
"""
from .continuous import ContinuousOperator
from .differentialOperator_d import DifferentialOperator_d


class DifferentialOperator(ContinuousOperator):
    """
    Differential operator
    """

    def __init__(self, field1, field2, choice=None):
        """
        Constructor.
        Create a Differential operator from given velocity and vorticity variables.

        @param field1 : velocity variable.
        @param field2 : vorticity variable.
        @param choice : choice of the differential operation in : curl , divWU , gradV, gradS
        """
        ContinuousOperator.__init__(self)
        self.field1 = field1
        self.field2 = field2
        self.choice = choice
        self.addVariable([field1, field2])

    def discretize(self, field1=None, field2=None, topology=None): 
        """
        Stretching operator discretization method.
        Create a discrete Stretching operator from given specifications.

        @param field1 : Index of velocity discretisation to use.
        @param field2 : Index of vorticity discretisation to use.
        @param topology : topology.
        """
        if self.discreteOperator is None:
            self.discreteOperator = DifferentialOperator_d(field1, field2, self.choice, topology)

    def __str__(self):
        """ToString method"""
        s = " Differencial operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : DifferentialOperator"
    print DifferentialOperator.__doc__
