# -*- coding: utf-8 -*-
"""
@package parmepy.operator Diffusion
Continous Diffusion operator (F2PY)
"""
from .continuous import ContinuousOperator
from .diffusion_d import Diffusion_d


class Diffusion(ContinuousOperator):
    """
    Diffusion operator representation using FFT
    """

    def __init__(self, velocity=None, vorticity=None, viscosity=0.1):
        """
        Constructor.
        Create a Diffusion operator using FFT.

        @param velocity ContinuousVectorField : velocity variable.
        @param vorticity ContinuousVectorField : vorticity variable.
        @param viscosity : viscosity of the considered medium.
        """
        ContinuousOperator.__init__(self)
        self.velocity = velocity
        self.vorticity = vorticity
        self.viscosity = viscosity
        self.addVariable([velocity, vorticity])

    def discretize(self, idVelocityD=0, idVorticityD=0, topology=None):
        """
        Diffusion operator discretization method.
        Create a discrete Diffusion operator from given specifications.

        @param idVelocityD : Index of velocity discretisation to use.
        @param idVelocityD : Index of vorticity discretisation to update.
        """
        if self.discreteOperator is None:
            self.discreteOperator = Diffusion_d(self, idVelocityD, idVorticityD, self.viscosity, topology)

    def __str__(self):
        """ToString method"""
        s = " Diffusion operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Diffusion"
    print Diffusion.__doc__
