# -*- coding: utf-8 -*-
"""
@package parmepy.operatorPressure

MultiPhase Rot Grad P
"""
from .continuous import ContinuousOperator
from .stretching_d import Stretching_d


class Pressure(ContinuousOperator):
    """
    Pressure operator representation
    """

    def __init__(self, velocity, vorticity, viscosity, density):
        """
        Constructor.
        Create a Pressure operator from given velocity variables.

        @param velocity ContinuousVectorField : velocity variable.
        """
        ContinuousOperator.__init__(self)
        ## velocity variable (vector)
        self.velocity = velocity
        self.vorticity = vorticity
        self.density = density
        self.viscosity = viscosity
        self.addVariable([velocity, vorticity, density])

    def discretize(self, topology=None, idVelocityD=0, idVorticityD=0, idDensityD=0):
        """
        Pressure operator discretization method.
        Create a discrete Pressure operator from given specifications.

        @param idVelocityD : Index of velocity discretisation to use.
        @param idVelocityD : Index of vorticity discretisation to update.
        @param idDensityD : Index of density discretisation to use.
        @param propertyOp : choice of the property garantied by the stretching Op ( divConservation or massConservation)
        @param method : the method to use. (Euler, RK, ..) default : RK3
        """
        if self.discreteOperator is None:
            self.discreteOperator = Pressure_d(self, topology, idVelocityD, idVorticityD, idDensityD)

    def __str__(self):
        """ToString method"""
        s = " Pressure operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Pressure"
    print Stretching.__doc__
