# -*- coding: utf-8 -*-
"""
@package operator
Discrete MultiPhase Rot Grad P
"""
from ..constants import *
from .discrete import DiscreteOperator
from .differentialOperator import DifferentialOperator
from .differentialOperator_d import DifferentialOperator_d
import numpy as np
import numpy.testing as npt
import time
import sys

class Pressure_d(DiscreteOperator):
    """
    Operator representation.
    """

    def __init__(self,pressureOp , topology=None, IdVelocityD=0, IdVorticityD=0, IdDensityD=0 ):
        """
        Constructor.
        Create the old velocity field to compute the -Dp/rho
        in N.S equation
        @param operator.
        """
        DiscreteOperator.__init__(self)
        ## Topology of the obstacle
        if(topology is not None):
            self.topology = topology
        else:
            raise NotImplementedError()
        self.velocity_old = np.copy(pressureOp.velocity.discreteField[IdVelocityD].data)
        self.velocity_new = pressureOp.velocity.discreteField[IdVelocityD]
        self.vorticity = pressureOp.vorticity.discreteField[IdVorticityD]
        self.density = pressureOp.density.discreteField[IdDensityD]
        self.resolution = self.topology.mesh.resolution
        self.meshSize = self.topology.mesh.size
        self.compute_time = 0.

    def apply(self, t, dt):
        """
        Apply operator.
        """
        self.compute_time = time.time()
        self.velocity_new = pressureOp.velocity.discreteField[IdVelocity]
        result = (self.velocity_new - self.velocity_old )/ dt
        gradU = DifferentialOperator_d(self.velocity_new, self.velocity_new, choice='gradV', topology=self.topology).apply()
        result = result + np.dot(self.velocity_new, gradU)
        ## result = - gradP/rho
        result = result - self.viscosity * DifferentialOperator_d(self.velocity_new, self.velocity_new, choice='laplacianV', topology=self.topology).apply()
        ## result = -(gradP/rho) x (gradRho/rho)
        result = np.cross(result, DifferentialOperator_d(self.density, self.density, choice='gradS', topology=self.topology).apply()/self.density)
        ## vorti(n+1) = vorti(n) + dt * result
        self.vorticity.data = self.vorticity.data + dt * result
        self.velocity_old = np.copy(self.velocity_new.data)
        self.compute_time = time.time() - self.compute_time
        return self.compute_time
#        return result

    def printComputeTime(self):
        self.timings_info[0] = "\"Pressure calculation total\""
#        self.timings_info[1] = str(self.total_time)
#        print "Multiphase total time : ", self.total_time
        print "Time of the last Dp/rho calculation loop :", self.compute_time 

    def __str__(self):
        s = "Pressure_d (DiscreteOperator). " + DiscreteOperator.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Pressure_d"
    print Pressure_d.__doc__

