# -*- coding: utf-8 -*-
"""
@package parmepy.operator.Penalization

Penalization operator representation
"""
from .continuous import ContinuousOperator
from penalization_d import Penalization_d


class Penalization(ContinuousOperator):
    """
    Penalization operator representation
    """

    def __init__(self, velocity, vorticity, obstacle, lambd):
        """
        Constructor.
        Create a Penalization operator from given velocity variavle.

        @param velocity ContinuousVariable.ContinuousVariable : velocity variable.
        """
        ContinuousOperator.__init__(self)
        ## obstacle to penalize
        self.obstacle = obstacle
        ## velocity variable to penalize
        self.velocity = velocity
        ## vorticity variable to penalize
        self.vorticity = vorticity
        ## lambda penalization function
        self.lambd = lambd
        self.addVariable([velocity, vorticity])

    def discretize(self, idVelocityD=0, idVorticityD=0, idObstacleD=0):
        """
        Penalization operator discretization method.
        Create a PenalizationDOp.PenalizationDOp from given specifications.

        """
        if self.discreteOperator is None:
            self.discreteOperator = Penalization_d(self, idVelocityD, idVorticityD, idObstacleD)

    def __str__(self):
        """ToString method"""
        s = " Penalization operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Penalization"
    print Penalization.__doc__
