# -*- coding: utf-8 -*-
"""
@package parmepy.operator Poisson solver
Continous Poisson solver operator (F2PY)
"""
from .continuous import ContinuousOperator
from .poisson_d import Poisson_d


class Poisson(ContinuousOperator):
    """
    Poisson solver operator representation using FFT
    """

    def __init__(self, velocity=None, vorticity=None):
        """
        Constructor.
        Create a Poisson solver operator using FFT.

        @param velocity ContinuousVectorField : velocity variable.
        @param vorticity ContinuousVectorField : vorticity variable.
        """
        ContinuousOperator.__init__(self)
        self.velocity = velocity
        self.vorticity = vorticity
        self.addVariable([velocity, vorticity])

    def discretize(self, idVelocityD=0, idVorticityD=0, topology=None):
        """
        Poisson solver operator discretization method.
        Create a discrete Poisson solver operator from given specifications.

        @param idVelocityD : Index of velocity discretisation to update from vorticity.
        @param idVelocityD : Index of vorticity.
        @param topology : topology of the input fields
        """
        if self.discreteOperator is None:
            self.discreteOperator = Poisson_d(self, idVelocityD, idVorticityD, topology)

    def __str__(self):
        """ToString method"""
        s = " Poisson solver operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Poisson"
    print Poisson.__doc__
