"""
@package parmepy.operator.transport

Transport operator representation.
"""
from .continuous import ContinuousOperator
from .transport_d import Transport_d


class Transport(ContinuousOperator):
    """
    Transport operator representation.
    """

    def __init__(self, velocity, scalar):
        """
        Create an Transport operator from given variables velocity and scalar.

        @param velocity : velocity variable.
        @param scalar : scalar variable.
        """
        ContinuousOperator.__init__(self)
        ## Transport velocity
        self.velocity = velocity
        ## Transported scalar
        self.scalar = scalar
        self.needSplitting = True
        self.addVariable([velocity, scalar])

    def discretize(self, idVelocityD=0, idScalarD=0, result_position=None, result_scalar=None, method=None):
        """
        Transport operator discretization method.
        Create an discrete Transport operator from given specifications.

        @param idVelocityD : Index of velocity discretisation to use.
        @param idScalarD : Index of scalar discretisation to use.
        @param result_position : result position.
        @param result_scalar : result scalar.
        @param method : the method to use.
        """
        if self.discreteOperator is None:
            self.discreteOperator = Transport_d(self, idVelocityD, idScalarD, result_position, result_scalar, method)

    def __str__(self):
        """ToString method"""
        s = "Advection operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += " with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Advection"
    print Advection.__doc__
