"""
@package parmepy.operator.velocity

Velocity operator representation.
"""
from .continuous import ContinuousOperator
from .velocity_d import Velocity_P


class Velocity(ContinuousOperator):
    """
    Velocity operator representation.
    """

    def __init__(self, velocity):
        """
        Create an operator for computing analyically a velocity field.

        @param velocity : velocity variable.
        """
        ContinuousOperator.__init__(self)
        ## velocity variable
        self.velocity = velocity
        self.addVariable([velocity])

    def discretize(self, idVelocityD=0, method=None):
        """
        Advection operator discretization method.
        Create an AdvectionDOp.AdvectionDOp from given specifications.

        @param idVelocityD : Index of velocity discretisation to use.
        @param method : the method to use.
        """
        if self.discreteOperator is None:
            self.discreteOperator = Velocity_P(self, idVelocityD, method)

    def __str__(self):
        """ToString method"""
        s = "Velocity operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += " with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Advection"
    print Advection.__doc__
