"""
@package parmepy.operator.velocity_d

Velocity operator representation.
"""
from ..constants import *
from .discrete import DiscreteOperator
import pyopencl as cl
import time


class Velocity_P(DiscreteOperator):
    """
    Velocity computation from analytical expression.x
    """

    def __init__(self, velocity_Op, idVelocityD=0, method=None):
        """
        Create a Velocity operator.

        @param velocity_Op : Velocity operator.
        @param idVelocityD : Index of velocity discretisation to use.
        @param method : the method to use.
        """
        DiscreteOperator.__init__(self)
        ## Velocity.
        self.velocity = velocity_Op.velocity.discreteField[idVelocityD]
        self.input = [self.velocity]
        self.output = [self.velocity]
        self.method = method
        self.needSplitting = False
        self.compute_time = 0.
        self.name = "velocity"
        self.gpu_precision = PARMES_REAL_GPU

    def apply(self, t, dt):
        c_time = 0.
        if self.numMethod is not None:
            # Compute velocity field
            dim = self.velocity.topology.dim
            coord_min = np.ones(4, dtype=self.gpu_precision)
            mesh_size = np.ones(4, dtype=self.gpu_precision)
            coord_min[0:dim] = self.velocity.topology.mesh.start
            mesh_size[0:dim] = self.velocity.topology.mesh.size
            if len(self.velocity.gpu_data) == 3:
                evt = self.numMethod.launch(self.velocity.gpu_data[0],
                                            self.velocity.gpu_data[1],
                                            self.velocity.gpu_data[2],
                                            self.gpu_precision(t),
                                            coord_min,
                                            mesh_size)
            else:
                evt = self.numMethod.launch(self.velocity.gpu_data[0],
                                            self.velocity.gpu_data[1],
                                            self.gpu_precision(t),
                                            coord_min,
                                            mesh_size)
            for df in self.output:
                df.contains_data = False
            self.numMethod.queue.finish()
            c_time = (evt.profile.end - evt.profile.start) * 1e-9
            self.compute_time += c_time
            self.total_time += c_time
        return c_time

    def printComputeTime(self):
        print "Velocity total time : ", self.total_time

    def __str__(self):
        s = "Velocity_P (DiscreteOperator). " + DiscreteOperator.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : AdvectionDOp"
    print AdvectionDOp.__doc__
