# -*- coding: utf-8 -*-
"""
@package Utils
Forward Euler method.
"""
from .integrator import ODESolver
import numpy as np


#providedClass = "Euler"


class Euler(ODESolver):
    """
    ODESolver implementation for solving an equation system with forward Euler method.
    y'(t) = f(t,y)

    y(t_{n+1}) = y(t_n) + dt*f(y(t_n))

    """
    def __init__(self, f=None, dim=1):
        """
        Constructor.

        @param f function f(t,y) : Right hand side of the equation to solve.
        @param dim : dimensions
        @param conditions : function to apply boundary conditions.
        """
        ODESolver.__init__(self, f)

    def integrate(self, f, t, dt, y):
        """
        Integration step for Euler Method.
         up = f[t, u(t,x,y,z)].

        @param u : function of position at time t.
        @param f : function of function of position at time t.
        @param t : current time.
        @param dt : time step.
#        """
        result = y + dt * np.asarray(f(t,y))
        return result


    def __str__(self):
        """ToString method"""
        return "Euler Method"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : " + providedClass
    print eval(providedClass).__doc__
