# -*- coding: utf-8 -*-
"""
@package Utils
ODESolver interface.
"""
from ...constants import *
import numpy as np

class ODESolver:
    """
    ODESolver interface for solving an equation system :
    y'(t) = f(t,y)

    By default, f = 0.
    """
    def __init__(self, f=None):
        """
        Constructor.

        @param f function f(t,y) : Right hand side of the equation to solve.
        """
        ## RHS.
        if f != None:
            self.f = f
        else:
            self.f = lambda t, y: np.vectorize(f)(t,y)# [0. for x in y]

    def integrate(self, y, t, dt):
        """
        Abstract method, apply operaton on a variable.
        Must be implemented by sub-class.

        @param y : position at time t.
        @param t : current time.
        @param dt : time step.
        @return y at t+dt.
        """
        raise NotImplementedError("Need to override method in a subclass of " + providedClass)

if __name__ == "__main__":
    print __doc__
    print "- Provided class : ODESolver"
    print ODESolver.__doc__
