# -*- coding: utf-8 -*-
"""
@package Utils
RK2 method interface.
"""
from .integrator import ODESolver
import numpy as np


class RK2(ODESolver):
    """
    ODESolver implementation for solving an equation system with RK2 method.
    y'(t) = f(t,y)

    y(t_n+1)= y(t_n) + dt*y'[y(t_n)+dt/2*y'(y(t_n))].

    """
    def __init__(self, f=None, dim=3):
        """
        Constructor.

        @param f function f(t,y) : Right hand side of the equation to solve.
        @param dim : dimensions
        """
        ODESolver.__init__(self, f)

    def integrate(self, f, t , dt , y):

        """
        Integration step for RK2 Method.
         up = f[t, Y(t,x,y,z)].

        @param y : function of position at time t.
        @param f : function of function of position at time t.
        @param t : current time.
        @param dt : time step.
        """
#K1 = dt *f[t,y(t,x,y,z)]
        K1 = dt * np.asarray(f(t,y))
#result = y(t,x,y,z)] + dt *f[t + dt/2 , y(t,x,y,z) + k1/2.]
        result = y + dt *np.asarray(f(t+dt/2.,y + K1/2.))
        return result

    def __str__(self):
        """ToString method"""
        return "RK2 Method"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : RK2"
    print RK2.__doc__
