"""
Module for testing parmepy.domain.topology
"""

import unittest
import parmepy as pp
import numpy as np


class CartesianTopologyTestCase(unittest.TestCase):
    def setUp(self):
        self.b = pp.Box()
        self.nbElem = [10, 10, 10]
        self.topo = pp.CartesianTopology(domain=self.b,
                                         resolution=self.nbElem,
                                         dim=len(self.nbElem))

    def tearDown(self):
        pass

    def test_constructor(self):
        self.assertEqual(self.topo.dim, len(self.nbElem))
        self.assertItemsEqual(self.topo.dims, np.asarray([1, 1, 1]))
        self.assertEqual(self.topo.size, 1)
        self.assertEqual(self.topo.rank, 0)


class LocalMeshTestCase(unittest.TestCase):
    def setUp(self):
        self.b = pp.Box()
        self.nbElem = [10, 10, 10]
        self.topo = pp.CartesianTopology(domain=self.b,
                                         resolution=self.nbElem,
                                         dim=len(self.nbElem))

    def tearDown(self):
        pass

    def test_constructor(self):
        self.assertItemsEqual(self.topo.mesh.origin, np.asarray([0., 0., 0.]))
        point = [self.topo.mesh.coords[0][0, 0, 0],
                 self.topo.mesh.coords[1][0, 0, 0],
                 self.topo.mesh.coords[2][0, 0, 0]]
        self.assertItemsEqual(np.asarray(point), np.asarray([0., 0., 0.]))
        point = [self.topo.mesh.coords[0][9, 0, 0],
                 self.topo.mesh.coords[1][0, 9, 0],
                 self.topo.mesh.coords[2][0, 0, 9]]
        self.assertItemsEqual(np.asarray(point), np.asarray([.9, .9, .9]))
        self.assertItemsEqual(self.topo.mesh.start, [0, 0, 0])
        self.assertItemsEqual(self.topo.mesh.end, [9, 9, 9])
        self.assertEqual(self.topo.mesh.coords[0].shape, (10, 1, 1))
        self.assertEqual(self.topo.mesh.coords[1].shape, (1, 10, 1))
        self.assertEqual(self.topo.mesh.coords[2].shape, (1, 1, 10))

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(
        unittest.TestLoader().loadTestsFromTestCase(LocalMeshTestCase)
        )
    unittest.TextTestRunner(verbosity=2).run(
        unittest.TestLoader().loadTestsFromTestCase(CartesianTopologyTestCase)
        )
