"""
Module for testing parmepy.field.analytical
"""

import unittest
import parmepy as pp
import numpy as np
from test_continuous import ContinuousFieldTestCase


class AnalyticalFieldTestCase(ContinuousFieldTestCase):
    def setUp(self):
        self.b = pp.Box()
        self.nbElem = [10, 10, 10]
        self.topo = pp.CartesianTopology(domain=self.b,
                                         resolution=self.nbElem,
                                         dim=len(self.nbElem))
        self.nbElem_bis = [10, 5, 5]
        self.topo_bis = pp.CartesianTopology(domain=self.b,
                                         resolution=self.nbElem_bis,
                                         dim=len(self.nbElem))
        self.field_scalar = pp.fields.analytical.AnalyticalField(self.b,
                                                                 formula=lambda x: x ** 3,
                                                                 name="test_field_scalar")
        self.field_vector = pp.fields.analytical.AnalyticalField(self.b,
                                                                 formula=lambda x, y, z: (x*y*z, 1.0, -x*y*z),
                                                                 name="test_field_vector", vector=True)

    def tearDown(self):
        pass

    def test_discretization_scalar(self):
        """Test discretization of a continuous scalar field
        - discretization with different topology
        - multiple discretizations"""
        ContinuousFieldTestCase.test_discretization_scalar(self)

    def test_discretization_vector(self):
        """Test discretization of a continuous vector field
        - discretization with different topology
        - multiple discretizations"""
        ContinuousFieldTestCase.test_discretization_vector(self)

    def test_values(self):
        """Test evaluation for different formulas"""
        self.assertEqual(self.field_scalar.value(0.), 0.)
        self.assertEqual(self.field_scalar.value(1.), 1.)
        self.assertEqual(self.field_scalar.value(-1.), -1.)
        self.assertEqual(self.field_vector.value(0., 0., 0.), (0., 1., 0.))
        self.assertEqual(self.field_vector.value(1., 1., 1.), (1., 1., -1.))
        self.assertEqual(self.field_vector.value(1., 2., 3.), (6., 1., -6.))


if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(
        unittest.TestLoader().loadTestsFromTestCase(AnalyticalFieldTestCase)
        )
