"""
Module for testing parmepy.field.continuous
"""

import unittest
import parmepy as pp
import numpy as np


class ContinuousFieldTestCase(unittest.TestCase):
    def setUp(self):
        self.b = pp.Box()
        self.nbElem = [10, 10, 10]
        self.topo = pp.CartesianTopology(domain=self.b,
                                         resolution=self.nbElem,
                                         dim=len(self.nbElem))
        self.nbElem_bis = [10, 5, 5]
        self.topo_bis = pp.CartesianTopology(domain=self.b,
                                         resolution=self.nbElem_bis,
                                         dim=len(self.nbElem))
        self.field_scalar = pp.fields.continuous.ContinuousField(self.b, "test_field_scalar")
        self.field_vector = pp.fields.continuous.ContinuousField(self.b, "test_field_vector", vector=True)

    def tearDown(self):
        pass

    def test_discretization_scalar(self):
        """Test discretization of a continuous scalar field
        - discretization with different topology
        - multiple discretizations"""
        self.assertEquals(self.field_scalar.discreteField, [])
        self.assertRaises(ValueError, self.field_scalar.initialize)
        df, df_id = self.field_scalar.discretize(self.topo)
        self.assertEqual(len(self.field_scalar.discreteField), 1)
        self.assertEqual(self.field_scalar.discreteField[df_id], df)
        self.assertTrue(isinstance(df, pp.fields.discrete.ScalarField))
        df, df_id = self.field_scalar.discretize(self.topo_bis)
        self.assertEqual(len(self.field_scalar.discreteField), 2)
        self.assertEqual(self.field_scalar.discreteField[df_id], df)
        self.assertTrue(isinstance(df, pp.ScalarField))

    def test_discretization_vector(self):
        """Test discretization of a continuous vector field
        - discretization with different topology
        - multiple discretizations"""
        self.assertEquals(self.field_vector.discreteField, [])
        self.assertRaises(ValueError, self.field_vector.initialize)
        df, df_id = self.field_vector.discretize(self.topo)
        self.assertEqual(len(self.field_vector.discreteField), 1)
        self.assertEqual(self.field_vector.discreteField[df_id], df)
        self.assertTrue(isinstance(df, pp.fields.discrete.VectorField))
        df, df_id = self.field_vector.discretize(self.topo_bis)
        self.assertEqual(len(self.field_vector.discreteField), 2)
        self.assertEqual(self.field_vector.discreteField[df_id], df)
        self.assertTrue(isinstance(df, pp.VectorField))

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(
        unittest.TestLoader().loadTestsFromTestCase(ContinuousFieldTestCase)
        )
