"""
@package parmepy.tools.profiling

External tool to profile parmepy problem solving.
"""
import argparse
import cProfile
import pstats
import os
import string
from meliae import scanner
from meliae import loader


# create Memory stats from file
def create_mem_stat(file):
    om = loader.load(file)
    summ = om.summarize()
    thesummary = loader._ObjSummary()
    objs = om.objs.itervalues()
    for obj in objs:
        thesummary._add(obj)
    if thesummary.summaries is None:
        thesummary.by_size()
    return thesummary


# New print stats function for pstats datas
def get_mem_stats(thesummary, amount=None):
    if amount == None:
        amount = len(thesummary.summaries)
    cumulative = 0
    for i in xrange(amount):
        summary = thesummary.summaries[i]
        cumulative += summary.total_size
        yield (i, summary.count, summary.count * 100.0 / thesummary.total_count, summary.total_size, summary.total_size * 100.0 / thesummary.total_size, cumulative * 100.0 / thesummary.total_size, summary.max_size, summary.type_str)


def print_mem_stat_file(thesummary, file):
    f = open(file, 'w')
    f.write("\n   Total {0} objects, {1} types, Total size = {2:.1f}MiB ({3} bytes)\n\n".format(memRawStats.total_count, len(memRawStats.summaries), memRawStats.total_size / 1024. / 1024, memRawStats.total_size))
    f.write(" Index    Count      %     Size      %    Cum      Max Kind \n")
    for l in get_mem_stats(memRawStats):
        f.write("{0:6d} {1:8d} {2:6.2f} {3:8d} {4:6.2f} {5:6.2f} {6:8d} {7:s}\n".format(l[0],l[1],l[2],l[3],l[4],l[5],l[6],l[7]))
    f.write("\n\n")
    f.close()


# New print stats function for pstats datas
def get_stats(stats, *amount):
    if stats.fcn_list:
        list = stats.fcn_list[:]
        msg = "   Ordered by: " + stats.sort_type + '\n'
    else:
        list = stats.stats.keys()
        msg = "   Random listing order was used\n"
    for selection in amount:
        list, msg = stats.eval_print_amount(selection, list, msg)
    if list:
        for func in list:
            cc, nc, tt, ct, callers = stats.stats[func]
            yield (cc, tt, ct, "%s:%d(%s)" % func)


def f8(x):
    return "%8.3f" % x


def print_line_file(stats, func, f):  # hack : should print percentages
        cc, nc, tt, ct, callers = stats.stats[func]
        c = str(nc)
        if nc != cc:
            c = c + '/' + str(cc)
        f.write(str(c.rjust(9)) + ' ')
        f.write(str(f8(tt)) + ' ')
        if nc == 0:
            f.write(str(' ' * 8) + ' ')
        else:
            f.write(str(f8(tt / nc)) + ' ')
        f.write(str(f8(ct)) + ' ')
        if cc == 0:
            f.write(str(' ' * 8) + ' ')
        else:
            f.write(str(f8(ct / cc)) + ' ')
        f.write("%s:%d(%s)" % func)
        f.write("\n")


def print_stats_file(stats, file, *amount):
    f = open(file, 'w')
    for filename in stats.files:
        f.write(str(filename))
    if stats.files:
        f.write("\n")
        indent = ' ' * 8
        for func in stats.top_level.keys():
            f.write(str(indent) + ' ' + str(func_get_function_name(func)))

        f.write(str(indent) + ' ' + str(stats.total_calls) + ' ' + str("function calls"))
        if stats.total_calls != stats.prim_calls:
            f.write(str("(%d primitive calls)" % stats.prim_calls))
        f.write(str("in %.3f CPU seconds" % stats.total_tt))
        f.write("\n")
        #width, list = stats.get_print_list(amount)
        width = stats.max_name_len
        if stats.fcn_list:
            list = stats.fcn_list[:]
            msg = "   Ordered by: " + stats.sort_type + '\n'
        else:
            list = stats.stats.keys()
            msg = "   Random listing order was used\n"

        for selection in amount:
            list, msg = stats.eval_print_amount(selection, list, msg)

        count = len(list)

        if not list:
            return 0, list
        f.write(msg)
        if count < len(stats.stats):
            width = 0
            for func in list:
                if  len(func_std_string(func)) > width:
                    width = len(func_std_string(func))
        if list:
            f.write('   ncalls  tottime  percall  cumtime  percall filename:lineno(function)\n')
            for func in list:
                print_line_file(stats, func, f)
        f.write("\n")
        f.write("\n")
        f.close()

# Argument parser creation
parser = argparse.ArgumentParser(description='Profile a given pyton script.')
parser.add_argument('-pyscript', required=True,
                   help='a python script to profile.')
parser.add_argument('-prefix', default='./',
                   help='output files prefix.')
parser.add_argument('-comm', default=" ",
                   help='Program description.')

# get arguments values
args = vars(parser.parse_args())
#script = __import__(args['pyscript'])
exec "from " + args['pyscript'] + " import *"
rawMemoryOutput = args['prefix']+args['pyscript'].replace('.','_')+'_profile_result.meliae'
MemorySizeOutput = args['prefix']+args['pyscript'].replace('.','_')+'_profile_result.meliae_size'
rawCProfileOutput = args['prefix']+args['pyscript'].replace('.','_')+'_profile_result.pstat'
cProfileTimeOutput = args['prefix']+args['pyscript'].replace('.','_')+'_profile_result.pstat_time'
cProfileCumulativeOutput = args['prefix']+args['pyscript'].replace('.','_')+'_profile_result.pstat_cumulative'
callTreePngOutput = args['prefix']+args['pyscript'].replace('.','_')+'_profile_calltree.png'
profileSummaryTex = args['prefix']+args['pyscript'].replace('.','_')+'_profile_summary.tex'
outputDir = callTreePngOutput.rsplit('/',1)[0]
callTreePngOutputFileName = callTreePngOutput.rsplit('/',1)[1]
profileSummaryTexFileName = profileSummaryTex.rsplit('/',1)[1]

run()

scanner.dump_all_objects(rawMemoryOutput)
memRawStats = create_mem_stat(rawMemoryOutput)
print_mem_stat_file(memRawStats, file=MemorySizeOutput)
memStat = "Total {0} objects, {1} types, Total size = {2:.1f}MiB ({3} bytes)\n".format(memRawStats.total_count, len(memRawStats.summaries), memRawStats.total_size / 1024. / 1024, memRawStats.total_size)
memStat += " Index    Count      %     Size      %    Cum      Max Kind \n"
memStatPercent = 0
memStatPieChart = ""
i = 0
for l in get_mem_stats(memRawStats, 10):
    memStat += "{0:6d} {1:8d} {2:6.2f} {3:8d} {4:6.2f} {5:6.2f} {6:8d} {7:s}\n".format(l[0],l[1],l[2],l[3],l[4],l[5],l[6],l[7])
    if i < 10:
        memStatPieChart += "{0:6.2f}/ {1:s} ".format(l[4], string.replace("\\texttt{"+l[7]+"}",'_','\_'))
        if i != 9:
            memStatPieChart += ","
        memStatPercent += l[4]
        i += 1
        #memStatPieChart += "{0:6.2f}/ {1:s}".format(100-memStatPercent, "\\texttt{Others}")


cProfile.run('run()', rawCProfileOutput)


p = pstats.Stats(rawCProfileOutput)
#p.strip_dirs().sort_stats('time').print_stats()
print_stats_file(p.strip_dirs().sort_stats('time'), file=cProfileTimeOutput)
print_stats_file(p.strip_dirs().sort_stats('cumulative'), file=cProfileCumulativeOutput)

# Calling gprof2dot to make graphic output calltree
os.system("gprof2dot -f pstats "+rawCProfileOutput+" | dot -Tpng -o "+callTreePngOutput)

# Create results string
dataFiles = ""
for filename in p.files:
    dataFiles += filename + ' '
cpuTotalTime = 'Total CPU time : ' + str(p.total_tt)
pstatTime = "  ncalls  tottime      %   cumtime     % filename:lineno(function) \n"
pstatTimePercent = 0
pstatTimeStringPieChart = ""
i = 0
for l in get_stats(p.strip_dirs().sort_stats('time'), 15):
    pstatTime+="{0:8d} {1:8.4f} {2:6.2f} {3:8.4f} {4:6.2f} {5:s}\n".format(l[0], l[1], 100*l[1]/p.total_tt, l[2], 100*l[2]/p.total_tt, l[3])
    if i < 10:
        pstatTimeStringPieChart += "{0:6.2f}/ {1:s} ".format(100*l[1]/p.total_tt, string.replace("\\texttt{"+l[3]+"}",'_','\_'))
        if i != 9:
            pstatTimeStringPieChart += ","
        pstatTimePercent += 100*l[1]/p.total_tt
        i += 1
        #pstatTimeStringPieChart += "{0:6.2f}/ {1:s}".format(100-pstatTimePercent, "\\texttt{Others}")

pstatCum = "  ncalls  tottime      %   cumtime     % filename:lineno(function) \n"
for l in get_stats(p.strip_dirs().sort_stats('cumulative'), 15):
    pstatCum+="{0:8d} {1:8.4f} {2:6.2f} {3:8.4f} {4:6.2f} {5:s}\n".format(l[0], l[1], 100*l[1]/p.total_tt, l[2], 100*l[2]/p.total_tt, l[3])


f = open(profileSummaryTex, 'w')
f.write("""\\documentclass[11pt]{scrartcl}
\\usepackage[top=1cm,bottom=1cm,left=1cm,right=1cm]{geometry}
\\usepackage{float,caption}
\\usepackage{pgf-pie}
\\usepackage{lscape}
\\usepackage{multicol}

\\begin{document}
\\begin{landscape}
\\begin{multicols}{2}
\\begin{verbatim}
"""
+ dataFiles
+ """
\\end{verbatim}

"""
+ args['comm']
+ """\\\\

"""
+ cpuTotalTime
+ """
\\section*{Top 15 CPU total time}

\\begin{footnotesize}
\\begin{verbatim}
"""
+ pstatTime
+ """
\\end{verbatim}
\\end{footnotesize}


\\section*{Top 15 CPU cumulative time}

\\begin{footnotesize}
\\begin{verbatim}
"""
+ pstatCum
+ """
\\end{verbatim}
\\end{footnotesize}

\\columnbreak
\\section*{Top 10 used memory}

\\begin{footnotesize}
\\begin{verbatim}
"""
+ memStat
+ """
\\end{verbatim}
\\end{footnotesize}

\\begin{figure}[H]
  %\\centering
\\begin{tikzpicture}[scale=0.75,font=\\footnotesize]
\\pie[explode=0,text=legend]{""" + pstatTimeStringPieChart + """}
\\end{tikzpicture}
  \\caption{Top 5 CPU total time}
\\end{figure}


\\begin{figure}[H]
  %\\centering
\\begin{tikzpicture}[scale=0.75,font=\\footnotesize]
\\pie[explode=0,text=legend]{""" + memStatPieChart + """}
\\end{tikzpicture}
  \\caption{Top 5 Used memory}
\\end{figure}

\\end{multicols}
\\end{landscape}
\\newpage
\\section*{Call graph}
\\begin{figure}[H]
  \\centering
  \\includegraphics[width=\\textwidth]{""" + callTreePngOutputFileName + """}
  \\caption{Call graph}
\\end{figure}


\\end{document}""")
f.close()

# Compiling LaTeX summary
cmd = "cd " + outputDir + ";"
cmd += "pdflatex -interaction batchmode " + profileSummaryTexFileName + ";"
cmd += "rm " + profileSummaryTexFileName.replace(".tex", ".aux") + ";"
cmd += "rm " + profileSummaryTexFileName.replace(".tex", ".log") + ";"
cmd += "cd -"
os.system(cmd)
