# -*- coding: utf-8 -*-
import unittest

from src.Problem.ContinuousTransportProblem import *
from src.Domain.Box import *
from src.Variable.AnalyticalVariable import *
from src.Operator.Advection import *
from src.Utils.ParticularSolver import *
#from src.Utils.ForwardEuler import *
from src.Utils.RK2 import *
from src.Utils.M6Prime import *
from src.Utils.Linear import *
from src.Utils.Printer import *
import numpy as np


class test_Gaussienne_Stationnaire_2D(unittest.TestCase):
    """
    Testing 2D problem with differents constant velocities
    """
    def setUp(self):
        self.e = 0.34  # Accepted error between result and analytical result
        self.dim = 2
        self.boxLength = [2., 2.]
        self.boxMin = [-1., -1.]
        self.nbElem = [100, 100]
        self.timeStep = 0.02
        self.box = Box(dimension=self.dim,
                       length=self.boxLength,
                       minPosition=self.boxMin)

    def solveAndTest(self):
        self.velo = AnalyticalVariable(domain=self.box,
                                   dimension=self.dim, formula=self._velocity)
        self.scal = AnalyticalVariable(domain=self.box,
                                   dimension=1, formula=self._scalarInit, scalar=True)
        self.advec = Advection(self.velo, self.scal)
        self.cTspPb = ContinuousTransportProblem(advection=self.advec)
        self.cTspPb.discretize(dSpec=[self.nbElem])
        InterpolationMethod = Linear(grid=self.box.discreteDomain, gvalues=self.velo.discreteVariable)
        RemeshingMethod = M6Prime(grid=self.box.discreteDomain, gvalues=self.scal.discreteVariable)
        ODESolver = RK2(InterpolationMethod, self.box.discreteDomain.applyConditions)
        solver = ParticularSolver(ODESolver, InterpolationMethod, RemeshingMethod)
        self.cTspPb.setSolver(solver)
        self.cTspPb.setPrinter(Printer(frequency=1, outputPrefix='res2D/RK2_GS_Complet_', problem=self.cTspPb))
        self.FinalTime = 1.
        goodScalar = np.array(self.scal.discreteVariable.values)
        self.cTspPb.solve(T=self.FinalTime, dt=self.timeStep)
        #norme = 0.
        #for i in self.box.discreteDomain.explore():
        #norme += (self._scalarInit(self.box.discreteDomain.points[i]) - self.scal.discreteVariable.values[i]) * (self._scalarInit(self.box.discreteDomain.points[i]) - self.scal.discreteVariable.values[i])
            #self.assertTrue(abs(self._scalarInit(p) - self.scal.discreteVariable.values[i]) < self.e)
            #[self.assertAlmostEqual(self.velo.discreteVariable.values[i][ii], self._velocity(p)[ii]) for ii in xrange(self.dim)]
            #print math.sqrt(norme)
        print np.linalg.norm(goodScalar - self.scal.discreteVariable.values)

    def testSolvingRK2_1(self):
        """Testing an advection around the box, v = (1,1) * 1/3 * dx"""
        self.v = (1. / 3.) * (self.boxLength[0] / (1. * self.nbElem[0]))
        self._velocity = self._velocityField
        self.solveAndTest()

    def _velocityField(self, x):
        return [- x[1] * 2 * 3.14159265, x[0] * 2 * 3.14159265]
        r = math.sqrt(x[0] * x[0] + x[1] * x[1])
        c = math.cos(3 * math.pi * r / 2.)
        return [-c * x[1], c * x[0]]

    def _scalarInit(self, x):
        r = math.sqrt(x[0] * x[0] + x[1] * x[1])
        if r < 1:
            return (1. - r * r) ** 6
        else:
            return 0.

    def runTest(self):
        self.setUp()
        self.testSolvingRK2_1()


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_Gaussienne_Stationnaire_2D))
    return suite

if __name__ == "__main__":
    #unittest.TextTestRunner(verbosity=2).run(suite())
    t = test_Gaussienne_Stationnaire_2D()
    t.setUp()
    t.testSolvingRK2_1()
