# -*- coding: utf-8 -*-
import unittest

from src.Domain.Box import *


class testBox(unittest.TestCase):
    """
    Box test class
    """
    def setUp(self):
        self.dim = 2
        self.min = [-1., -1.]
        self.length = [3, 3]
        self.max = [2., 2.]
        self.defaultBox = Box()
        self.box = Box(dimension=self.dim, minPosition=self.min, length=self.length)

    def testValues(self):
        """--> Box initialisation testing"""
        self.assertEqual(self.defaultBox.dimension, 1)
        self.assertEqual(self.defaultBox.length[0], 1)
        self.assertEqual(self.defaultBox.min[0], 0.0)
        self.assertEqual(self.defaultBox.max[0], 1.0)
        self.assertEqual(self.box.dimension, self.dim)
        for i in xrange(2):
            self.assertEqual(self.box.length[i], self.length[i])
            self.assertEqual(self.box.min[i], self.min[i])
            self.assertEqual(self.box.max[i], self.max[i])

    def testDiscretization(self):
        """--> Testing Box discretization into Grid"""
        self.box.discretize([3, 3])
        for i in self.box.discreteDomain.explore():
            self.assertEqual(self.box.discreteDomain.points[i][0], -1. + i[0] * 1.)
            self.assertEqual(self.box.discreteDomain.points[i][1], -1 + i[1] * 1.)

    def testExceptionsDimensions(self):
        """--> Exception raising for inconsistent Box parameters dimensions"""
        try:
            Box(self.dim - 1, self.length, self.min)
        except ValueError:
            pass
        else:
            self.fail("Exception raised when Box parameters dimensions are not equal")


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testBox))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
