# -*- coding: utf-8 -*-
import unittest

from src.Operator.Advection import *

from src.Domain.Box import *
from src.Variable.AnalyticalVariable import *


class testAdvection(unittest.TestCase):
    """
    Advection operator test class
    """
    def setUp(self):
        dim = 1
        self.b = Box(dimension=dim, length=[1.], minPosition=[0.])
        self.velocityField = AnalyticalVariable(domain=self.b, dimension=1, formula=self._vitesse)
        self.initialScalar = AnalyticalVariable(domain=self.b, dimension=1, formula=self._scalaire)
        self.tsp = Advection(velocity=self.velocityField, scalar=self.initialScalar)

    def testDiscretization(self):
        """--> Testing discretization of Transport Operator """
        self.b.discretize([11])
	self.velocityField.discretize()
	self.initialScalar.discretize()
        self.tsp.discretize()

    def _vitesse(self, x):
        return [1.]

    def _scalaire(self, x):
        if x[0] < 0.5:
            return 1.
        else:
            return 0.


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testAdvection))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
