# -*- coding: utf-8 -*-
import unittest

from src.Problem.ContinuousTransportProblem import *
from src.Domain.Box import *
from src.Variable.AnalyticalVariable import *
from src.Operator.Advection import *
from src.Utils.ForwardEuler import *
from src.Utils.Lambda1 import *
from src.Utils.Linear import *
from src.Utils.ParticularSolver import *
import numpy as np


class testInterpolationDOp(unittest.TestCase):
    """
    InterpolationDOp test class
    """
    def setUp(self):
        self.nbElem = {}
        self.box = {}
        self.velo = {}
        self.InterpolationMethod = {}
        self.parts = {}
        self.ppos = {}
        self.pvelo = {}
        self.interpol = {}
        self.dim = [1, 2, 3]
        for dim in self.dim:
            self.nbElem[dim] = [10 for i in xrange(dim)]
            self.box[dim] = Box(dimension=dim, length=[1. for i in xrange(dim)], minPosition=[0. for i in xrange(dim)])
            self.velo[dim] = AnalyticalVariable(domain=self.box[dim], dimension=dim, formula=self._vitesse)
            self.box[dim].discretize(spec=self.nbElem[dim])
            self.velo[dim].discretize()
            self.InterpolationMethod[dim] = Linear(grid=self.box[dim].discreteDomain, gvalues=self.velo[dim].discreteVariable)
            self.parts[dim] = ParticleField(self.box[dim].discreteDomain)
            self.ppos[dim] = DiscreteVariable(domain=self.parts[dim], dimension=dim)
            self.parts[dim].setPositions(self.ppos[dim])
            self.pvelo[dim] = DiscreteVariable(domain=self.parts[dim], dimension=dim)
            self.interpol[dim] = InterpolationDOp(position=self.ppos[dim],
                                                  velocityField=self.velo[dim].discreteVariable,
                                                  resvelo=self.pvelo[dim],
                                                  method=self.InterpolationMethod[dim])

    def testApplyOperatorOnGridPoints(self):
        """--> Testing Apply Interpolation Operator on grid points"""
        for dim in self.dim:
            ones = np.ones(dim)
            # Particules on grid points
            for i in self.box[dim].discreteDomain.explore():
                self.ppos[dim].values[i] = [x for x in self.box[dim].discreteDomain.points[i]]
            # Interpolation
            for d in xrange(dim):
                self.interpol[dim].applyOperator(t=1., dt=1., splittingDirection=d)
                # Tests : particles velocity = grid velocity
                for i in self.parts[dim].explore():
                    self.assertTrue((ones - self.pvelo[dim].values[i][d] < 0.00000001).all())

    def testApplyOperatorAdvectedParticles(self):
        """--> Testing Apply Interpolation Operator on advected particles (particles on mid-grid points) along x axis"""
        for dim in self.dim:
            ones = np.ones(dim)
            # Particules on mid-grid points
            for i in self.box[dim].discreteDomain.explore():
                self.ppos[dim].values[i] = [self.box[dim].discreteDomain.points[i][d] + 0.75 / self.nbElem[dim][d] for d in xrange(dim)]
            # Interpolation
            for d in xrange(dim):
                self.interpol[dim].applyOperator(t=1., dt=1., splittingDirection=d)
                for i in self.parts[dim].explore():
                    self.assertTrue((ones - self.pvelo[dim].values[i][d] < 0.00000001).all())




    def _vitesse(self, x):
        return np.ones_like(x)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testInterpolationDOp))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
