# -*- coding: utf-8 -*-
import unittest

from src.Domain.Box import *
from src.Variable.DiscreteVariable import *
from src.Domain.ParticleField import *
from src.Operator.TagDOp import *


class testTagDOp(unittest.TestCase):
    """
    TagDOp test class
    """
    def setUp(self):
        self.nbElem = 10
        self.box = Box(dimension=1, length=[1.], minPosition=[0.])
        self.box.discretize(spec=[self.nbElem])
        self.parts = ParticleField(self.box.discreteDomain)
        self.ppos = DiscreteVariable(domain=self.parts, dimension=1)
        self.parts.setPositions(self.ppos)
        self.pvelo = DiscreteVariable(domain=self.parts, dimension=1)
        self.pbloc = DiscreteVariable(domain=self.parts, dimension=1, scalar=True, integer=True)
        self.ptag = DiscreteVariable(domain=self.parts, dimension=1, scalar=True, integer=True)
        self.tag = TagDOp(partPositions=self.ppos,
                          partVelocities=self.pvelo,
                          grid=self.box.discreteDomain,
                          resbloc=self.pbloc,
                          restag=self.ptag)

    def testApplyOperator_CaseA(self):
        """--> Testing Apply Tag Operatror with hand-made velocity for article's cases a and a'"""
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            if i[0] == 0:
                self.pvelo.values[i] = [-0.9]
            elif i[0] in [1, 8, 9]:
                self.pvelo.values[i] = [0.1]
            else:
                self.pvelo.values[i] = [-0.1]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Tag des particules 1, 2, 7 et 8
        # Bloc Gauche pour les particules 0, 1, 8 et 9, Centré sinon.
        [self.assertEqual(self.pbloc.values[i], 0) if i[0] in [0, 1, 8, 9] else self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertNotEqual(self.ptag.values[i], 0) if i[0] in [1, 2, 7, 8] else self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_CaseB(self):
        """--> Testing Apply Tag Operatror with hand-made velocity for article's cases b and b'"""
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            if i[0] in [0, 2, 7, 9]:
                self.pvelo.values[i] = [0.1]
            elif i[0] in [1, 8]:
                self.pvelo.values[i] = [0.9]
            else:
                self.pvelo.values[i] = [-0.1]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Tag des particules 1, 2, 7 et 8
        # Bloc Gauche pour les particules 0, 1, 8 et 9, Centré sinon.
        [self.assertEqual(self.pbloc.values[i], 0) if i[0] in [0, 1, 8, 9] else self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertNotEqual(self.ptag.values[i], 0) if i[0] in [1, 2, 7, 8] else self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_CaseC(self):
        """--> Testing Apply Tag Operatror with hand-made velocity for article's cases c and c'"""
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            if i[0] in [0]:
                self.pvelo.values[i] = [0.1]
            elif i[0] in [5]:
                self.pvelo.values[i] = [0.55]
            elif i[0] in [6]:
                self.pvelo.values[i] = [-0.05]
            elif i[0] in [8, 9]:
                self.pvelo.values[i] = [0.1]
            else:
                self.pvelo.values[i] = [0.9]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Tag des particules 1, 2, 7 et 8
        # Bloc Gauche pour les particules 0, 1, 8 et 9, Centré sinon.
        [self.assertEqual(self.pbloc.values[i], 0) if i[0] in [0, 1, 8, 9] else self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertNotEqual(self.ptag.values[i], 0) if i[0] in [1, 2, 7, 8] else self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_CaseD(self):
        """--> Testing Apply Tag Operatror with hand-made velocity for article's cases d and d'"""
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            if i[0] in [0, 7]:
                self.pvelo.values[i] = [0.1]
            elif i[0] in [2]:
                self.pvelo.values[i] = [1.1]
            elif i[0] in [8]:
                self.pvelo.values[i] = [-0.1]
            elif i[0] in [9]:
                self.pvelo.values[i] = [-0.9]
            else:
                self.pvelo.values[i] = [0.9]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Tag des particules 1, 2, 7 et 8
        # Bloc Gauche pour les particules 0, 1, 8 et 9, Centré sinon.
        [self.assertEqual(self.pbloc.values[i], 0) if i[0] in [0, 1, 8, 9] else self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertNotEqual(self.ptag.values[i], 0) if i[0] in [1, 2, 7, 8] else self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_v01(self):
        """--> Testing Apply Tag Operator with velocity=0.1 and clf=1"""
        v = 0.1
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Gauche (0)
        [self.assertEqual(self.pbloc.values[i], 0) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_v09(self):
        """--> Testing Apply Tag Operator with velocity=0.9 and clf=1"""
        v = 0.9
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Centré (1)
        [self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_vm01(self):
        """--> Testing Apply Tag Operator with velocity=-0.1 and clf=1"""
        v = -0.1
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Centré (1)
        [self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_vm09(self):
        """--> Testing Apply Tag Operator with velocity=-0.9 and clf=1"""
        v = -0.9
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Gauche (0)
        [self.assertEqual(self.pbloc.values[i], 0) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_v51(self):
        """--> Testing Apply Tag Operator with velocity=0.1 and clf=1"""
        v = 5.1
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Gauche (0)
        [self.assertEqual(self.pbloc.values[i], 0) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_v59(self):
        """--> Testing Apply Tag Operator with velocity=0.9 and clf=1"""
        v = 5.9
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Centré (1)
        [self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_vm51(self):
        """--> Testing Apply Tag Operator with velocity=-0.1 and clf=1"""
        v = -5.1
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Centré (1)
        [self.assertEqual(self.pbloc.values[i], 1) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]

    def testApplyOperator_vm59(self):
        """--> Testing Apply Tag Operator with velocity=-0.9 and clf=1"""
        v = -5.9
        cfl = 1.
        dt = cfl * self.box.discreteDomain.elementSize[0]
        # Initialisation des positions et vitesses
        for i in self.parts.explore():
            self.pvelo.values[i] = [v]
            self.ppos.values[i] = [i[0] * self.box.discreteDomain.elementSize[0] + dt * self.pvelo.values[i][0]]
        self.tag.applyOperator(t=0., dt=dt, splittingDirection=0)
        # Pas de tag. Toutes les particules appartiennent à des blocs Gauche (0)
        [self.assertEqual(self.pbloc.values[i], 0) for i in self.parts.explore()]
        [self.assertEqual(self.ptag.values[i], 0) for i in self.parts.explore()]


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testTagDOp))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
