# -*- coding: utf-8 -*-
import unittest

from src.Problem.ContinuousTransportProblem import *
from src.Domain.Box import *
from src.Variable.AnalyticalVariable import *
from src.Operator.Advection import *
from src.Utils.ForwardEuler import *
from src.Utils.Lambda2 import *
from src.Utils.Linear import *
from src.Utils.ParticularSolver import *


class testContinuousTransportProblem(unittest.TestCase):
    """
   ContinuousTransportProblem test class
    """
    def setUp(self):
        pass

    def test1DProblem(self):
        """--> Testing advection Problem construction"""
        dim = 1
        nbElem = 32
        box = Box(dimension=dim, length=[1.], minPosition=[0.])
        velocityField = AnalyticalVariable(domain=box, dimension=1, formula=self._vitesse1D)
        initialScalar = AnalyticalVariable(domain=box, dimension=1, formula=self._scalaire1D)
        tsp = Advection(velocity=velocityField, scalar=initialScalar)
        tspPb = ContinuousTransportProblem(tsp)
        tspPb.addDomain([box])
        self.assertEqual(len(tspPb.domains), 1)
        self.assertEqual(len(tspPb.variables), 2)
        self.assertEqual(len(tspPb.operators), 1)
        # Ajout du domaine
        tspPb.addDomain([box])
        self.assertEqual(len(tspPb.domains), 1)
        self.assertEqual(len(tspPb.variables), 2)
        self.assertEqual(len(tspPb.operators), 1)
        tspPb.addVariable([velocityField, initialScalar])
        tspPb.addVariable([velocityField, initialScalar])
        self.assertEqual(len(tspPb.domains), 1)
        self.assertEqual(len(tspPb.variables), 2)
        self.assertEqual(len(tspPb.operators), 1)
        tspPb.addOperator([tsp])
        self.assertEqual(len(tspPb.domains), 1)
        self.assertEqual(len(tspPb.variables), 2)
        self.assertEqual(len(tspPb.operators), 1)
        tspPb.discretize(dSpec=[[nbElem]])
        # discrétisation des éléments continus
        self.assertEqual(len(tspPb.domains), 1)
        self.assertEqual(len(tspPb.variables), 2)
        self.assertEqual(len(tspPb.operators), 1)
        self.assertEqual(len(tspPb.discreteProblem.domains), 1)
        self.assertEqual(len(tspPb.discreteProblem.variables), 2)
        self.assertEqual(len(tspPb.discreteProblem.operators), 1)
        InterpolationMethod = Linear(grid=box.discreteDomain, gvalues=velocityField.discreteVariable)
        RemeshingMethod = Lambda2(grid=box.discreteDomain, gvalues=initialScalar.discreteVariable)
        ODESolver = ForwardEuler(InterpolationMethod)
        solver = ParticularSolver(ODESolver, InterpolationMethod, RemeshingMethod)
        tspPb.setSolver(solver)
        # Création des variables de particules : position, vitesse, scalaire, tag et bloc
        # Création du domaine des particles
        # Création des 4 opérateurs : advection, interpolation, tag et remaillage
        self.assertEqual(len(tspPb.domains), 1)
        self.assertEqual(len(tspPb.variables), 2)
        self.assertEqual(len(tspPb.operators), 1)
        self.assertEqual(len(tspPb.discreteProblem.domains), 2)
        self.assertEqual(len(tspPb.discreteProblem.variables), 7)
        self.assertEqual(len(tspPb.discreteProblem.operators), 4)

    def _vitesse1D(self, x):
        return [1.]

    def _scalaire1D(self, x):
        if x[0] < 0.5:
            return 1.
        else:
            return 0.


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testContinuousTransportProblem))

    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
