# -*- coding: utf-8 -*-
import unittest

from src.Problem.ContinuousTransportProblem import *
from src.Domain.Box import *
from src.Variable.AnalyticalVariable import *
from src.Operator.Advection import *
from src.Utils.ForwardEuler import *
from src.Utils.Lambda2 import *
from src.Utils.Linear import *
from src.Utils.ParticularSolver import *


class testDiscreteTransportProblem(unittest.TestCase):
    """
   DiscreteTransportProblem test class
    """
    def setUp(self):
        pass

    def testSolving1DProblem(self):
        """--> Testing advection at constant velocity in one time step"""
        dim = 1
        nbElem = 32
        box = Box(dimension=dim, length=[1.], minPosition=[0.])
        velocityField = AnalyticalVariable(domain=box, dimension=1, formula=self._vitesse1D)
        initialScalar = AnalyticalVariable(domain=box, dimension=1, formula=self._scalaire1D)
        tsp = Advection(velocity=velocityField, scalar=initialScalar)
        tspPb = ContinuousTransportProblem(tsp)
        tspPb.discretize(dSpec=[[nbElem]])
        InterpolationMethod = Linear(grid=box.discreteDomain, gvalues=velocityField.discreteVariable)
        RemeshingMethod = Lambda2(grid=box.discreteDomain, gvalues=initialScalar.discreteVariable)
        ODESolver = ForwardEuler(InterpolationMethod)
        solver = ParticularSolver(ODESolver, InterpolationMethod, RemeshingMethod)
        tspPb.setSolver(solver)
        tspPb.solve(T=0.75 / 32., dt=0.75 / 32.)
        for i in box.discreteDomain.explore():
            if i[0] == 0:
                self.assertAlmostEqual(initialScalar.discreteVariable.values[i], 0.15625)
            elif i[0] == 1:
                self.assertAlmostEqual(initialScalar.discreteVariable.values[i], 1.09375)
            elif i[0] == 16:
                self.assertAlmostEqual(initialScalar.discreteVariable.values[i], 0.84375)
            elif i[0] == 17:
                self.assertAlmostEqual(initialScalar.discreteVariable.values[i], -0.09375)
            elif i[0] < 16:
                self.assertAlmostEqual(initialScalar.discreteVariable.values[i], 1.)
            else:
                self.assertAlmostEqual(initialScalar.discreteVariable.values[i], 0.)

    def _vitesse1D(self, x):
        return [1.]

    def _scalaire1D(self, x):
        if x[0] < 0.5:
            return 1.
        else:
            return 0.


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testDiscreteTransportProblem))

    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
