# -*- coding: utf-8 -*-
import unittest

#import parmepy

from parmepy.operator.transport import *
from parmepy.operator.continuous import *
from parmepy.operator.differentialOperator import *
from parmepy.operator.stretching import *
from parmepy.fields.discrete import *
from parmepy.fields.continuous import *
from parmepy.fields.analytical import *
from parmepy.domain.topology import *
from parmepy.domain.box import *
from parmepy.constants import *
from parmepy.particular_solvers.basic import *
from parmepy.particular_solvers.integrator.euler import *
from parmepy.particular_solvers.solver import *
import numpy as np
import numpy.testing as npt
import math


class test_DivProduct(unittest.TestCase):
    """
    DiscreteVariable test class
    """
    def setUp(self):
        self.e = 0.2  # Accepted error between result and analytical result
        self.dim = 3
        self.boxLength = [2.*np.pi, 2.*np.pi, 2.*np.pi]
        self.boxMin = [ 0., 0., 0.]
        self.nbPts = [32, 32, 32]
        self.timeStep = 0.02
        self.box = Box(dimension=self.dim,
                       length=self.boxLength,
                       origin=self.boxMin)

    def testOperatorDiv(self):
        # Continuous fields and operator declaration
        self.velo = AnalyticalField(domain=self.box, formula=self.vitesse, name='Velocity', vector=True)
        self.vorti = AnalyticalField(domain=self.box, formula=self.vorticite, name='Vorticity', vector=True)
        self.div = DifferentialOperator(self.vorti, self.velo, choice='divergenceProduct')
        # Topology definition / Fields and operator discretization
        self.result = [np.ones((self.nbPts), dtype=PARMES_REAL, order=ORDER) for d in xrange(self.dim)]
        self.topo3D = CartesianTopology(domain=self.box, resolution=self.nbPts, dim=self.dim)
        self.vorti.discretize(self.topo3D)
        self.velo.discretize(self.topo3D)
        self.vorti.initialize()
        self.velo.initialize()
        self.div.discretize(self.vorti.discreteField[self.vorti._fieldId], self.velo.discreteField[self.velo._fieldId], result=self.result)
        self.div.discreteOperator.apply()
        self.resol = self.topo3D.mesh.resolution
        self.FinalTime = 0.2
        self.anal=self.analyticalDivProduct(self.topo3D.mesh.coords[0], \
                                                     self.topo3D.mesh.coords[1], \
                                                     self.topo3D.mesh.coords[2])
        # Comparison with analytical solution 
        npt.assert_array_less(abs(self.anal[0]-self.result[0]), self.e)
        npt.assert_array_less(abs(self.anal[1]-self.result[1]), self.e)
        npt.assert_array_less(abs(self.anal[2]-self.result[2]), self.e)

    def vitesse(self, x, y, z):
        vx = np.sin(x)
        vy = np.sin(y)
        vz = np.sin(z)
        return vx, vy, vz

    def vorticite(self, x, y, z):
        wx = 1.
        wy = 1.
        wz = 1.
        return wx, wy, wz

    def analyticalDivProduct(self, x, y, z):
        sx = np.cos(x)+ 0.*y*z
        sy = np.cos(y)+ 0.*x*z
        sz = np.cos(z)+ 0.*x*y
        return sx, sy, sz

    def runTest(self):
        self.setUp()
        self.testOperatorDiv()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_DivProduct))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
