# -*- coding: utf-8 -*-
import unittest

from src.Variable.AnalyticalVariable import *

from src.Domain.Box import *


def formula1D(position):
    """Fonction test 1D : f(x) = x**2  """
    return [position[0] * position[0]]


def formula31D(position):
    """Fonction test 1D : f(x,y,z) = x*y*z  """
    return [position[0] * position[1] * position[2]]


def formula2D(position):
    """Fonction test 2D : f(x,y) = (x - y,x + y) """
    return [position[0] - position[1], position[0] + position[1]]


def formula3D(position):
    """Fonction test 3D : f(x,y,z) = (y - x,-xz - y,xy - z) """
    return [position[1] - position[0], -position[0] * position[2] - position[1], position[0] * position[1] - position[2]]


class testAnalyticalVariable(unittest.TestCase):
    """
    DiscreteVariable test class
    """
    def setUp(self):
        self.b1D = Box(1, [1.], [0.])
        self.b2D = Box(2, [1., 1.], [0., 0.])
        self.b3D = Box(3, [1., 1., 1.], [0., 0., 0.])
        self.v1D = AnalyticalVariable(self.b1D, 1, formula1D)
        self.v2D = AnalyticalVariable(self.b2D, 2, formula2D)
        self.v3D = AnalyticalVariable(self.b3D, 3, formula3D)
        self.v31D = AnalyticalVariable(self.b3D, 1, formula31D)

    def testValue1D(self):
        """--> Test fonction 1D : f(x) = x**2  """
        self.assertAlmostEqual(self.v1D.value([2.])[0], 4.)

    def testValue2D(self):
        """--> Test fonction 2D : f(x,y) = (x - y,x + y) """
        self.assertAlmostEqual(self.v2D.value([1., 2.])[0], -1.)
        self.assertAlmostEqual(self.v2D.value([1., 2.])[1], 3.)

    def testValue3D(self):
        """--> Test fonction 3D : f(x,y,z) = (y - x,-xz - y,xy - z) """
        self.assertAlmostEqual(self.v3D.value([1., 2., 3.])[0], 1.)
        self.assertAlmostEqual(self.v3D.value([1., 2., 3.])[1], -5.)
        self.assertAlmostEqual(self.v3D.value([1., 2., 3.])[2], -1.)

    def testValue31D(self):
        """--> Test fonction 3D : f(x,y,z) = (y - x,-xz - y,xy - z) """
        self.assertAlmostEqual(self.v31D.value([1., 2., 3.])[0], 6.)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testAnalyticalVariable))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
