# -*- coding: utf-8 -*-
import unittest

from src.Domain.Box import *
from src.Variable.AnalyticalVariable import *


class testDiscreteVariable(unittest.TestCase):
    """
    DiscreteVariable test class
    """
    def setUp(self):
        self.box = Box(dimension=3, length=[1., 1., 1.], minPosition=[0., 0., 0.])
        self.velo = AnalyticalVariable(domain=self.box, dimension=3, formula=self._vitesse)
        self.scal = AnalyticalVariable(domain=self.box, dimension=1, formula=self._scalaire)
        self.box.discretize([10, 10, 10])
        self.velo.discretize()
        self.scal.discretize()

    def testVariables(self):
        """--> Test discrete Variables."""
        self.assertEqual(self.velo.discreteVariable.values.shape, (10, 10, 10, 3))
        self.assertEqual(self.scal.discreteVariable.values.shape, (10, 10, 10, 1))
        self.assertEqual(len(self.velo.discreteVariable.values[(0, 0, 0)]), 3)
        self.assertEqual(len([self.scal.discreteVariable.values[(0, 0, 0)]]), 1)
        index = xrange(3)
        for ii in self.box.discreteDomain.explore():
            [self.assertAlmostEqual(self.velo.discreteVariable.values[ii][i], self._vitesse(self.box.discreteDomain.points[ii])[i]) for i in index]
            self.assertAlmostEqual(self.scal.discreteVariable.values[ii], self._scalaire(self.box.discreteDomain.points[ii]))

    def _vitesse(self, x):
        return [1., 1., 1.]

    def _scalaire(self, x):
        if x[0] < 0.5 and x[1] < 0.5 and x[2] < 0.5:
            return 1.
        else:
            return 0.


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testDiscreteVariable))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
